
(install
 (section share)
 (files
   (ANSI_X3%2E110-1983.mar as charmaps/ANSI_X3%2E110-1983.mar)
   (NAPLPS.mar as charmaps/NAPLPS.mar)
   (CSA_T500-1983.mar as charmaps/CSA_T500-1983.mar)
   (ISO-IR-99.mar as charmaps/ISO-IR-99.mar)
   (ANSI_X3%2E4-1968.mar as charmaps/ANSI_X3%2E4-1968.mar)
   (CP367.mar as charmaps/CP367.mar)
   (IBM367.mar as charmaps/IBM367.mar)
   (US.mar as charmaps/US.mar)
   (US-ASCII.mar as charmaps/US-ASCII.mar)
   (ISO646-US.mar as charmaps/ISO646-US.mar)
   (ASCII.mar as charmaps/ASCII.mar)
   (ISO_646%2EIRV%3A1991.mar as charmaps/ISO_646%2EIRV%3A1991.mar)
   (ANSI_X3%2E4-1986.mar as charmaps/ANSI_X3%2E4-1986.mar)
   (ISO-IR-6.mar as charmaps/ISO-IR-6.mar)
   (ARMSCII-8.mar as charmaps/ARMSCII-8.mar)
   (ASMO_449.mar as charmaps/ASMO_449.mar)
   (ISO-IR-89.mar as charmaps/ISO-IR-89.mar)
   (ARABIC7.mar as charmaps/ARABIC7.mar)
   (ISO_9036.mar as charmaps/ISO_9036.mar)
   (BIG5.mar as charmaps/BIG5.mar)
   (BIG5-CP950.mar as charmaps/BIG5-CP950.mar)
   (BIG5-HKSCS.mar as charmaps/BIG5-HKSCS.mar)
   (BIG5HKSCS.mar as charmaps/BIG5HKSCS.mar)
   (BRF.mar as charmaps/BRF.mar)
   (BS_4730.mar as charmaps/BS_4730.mar)
   (UK.mar as charmaps/UK.mar)
   (GB.mar as charmaps/GB.mar)
   (ISO646-GB.mar as charmaps/ISO646-GB.mar)
   (ISO-IR-4.mar as charmaps/ISO-IR-4.mar)
   (BS_VIEWDATA.mar as charmaps/BS_VIEWDATA.mar)
   (ISO-IR-47.mar as charmaps/ISO-IR-47.mar)
   (CP10007.mar as charmaps/CP10007.mar)
   (CP1125.mar as charmaps/CP1125.mar)
   (IBM848.mar as charmaps/IBM848.mar)
   (RUSCII.mar as charmaps/RUSCII.mar)
   (CP1250.mar as charmaps/CP1250.mar)
   (MS-EE.mar as charmaps/MS-EE.mar)
   (CP1251.mar as charmaps/CP1251.mar)
   (MS-CYRL.mar as charmaps/MS-CYRL.mar)
   (CP1252.mar as charmaps/CP1252.mar)
   (MS-ANSI.mar as charmaps/MS-ANSI.mar)
   (CP1253.mar as charmaps/CP1253.mar)
   (MS-GREEK.mar as charmaps/MS-GREEK.mar)
   (CP1254.mar as charmaps/CP1254.mar)
   (MS-TURK.mar as charmaps/MS-TURK.mar)
   (CP1255.mar as charmaps/CP1255.mar)
   (MS-HEBR.mar as charmaps/MS-HEBR.mar)
   (CP1256.mar as charmaps/CP1256.mar)
   (MS-ARAB.mar as charmaps/MS-ARAB.mar)
   (CP1257.mar as charmaps/CP1257.mar)
   (WINBALTRIM.mar as charmaps/WINBALTRIM.mar)
   (CP1258.mar as charmaps/CP1258.mar)
   (CP737.mar as charmaps/CP737.mar)
   (CP775.mar as charmaps/CP775.mar)
   (CP932.mar as charmaps/CP932.mar)
   (CP949.mar as charmaps/CP949.mar)
   (CSA_Z243%2E4-1985-1.mar as charmaps/CSA_Z243%2E4-1985-1.mar)
   (CA.mar as charmaps/CA.mar)
   (CSA7-1.mar as charmaps/CSA7-1.mar)
   (ISO646-CA.mar as charmaps/ISO646-CA.mar)
   (ISO-IR-121.mar as charmaps/ISO-IR-121.mar)
   (CSA_Z243%2E4-1985-2.mar as charmaps/CSA_Z243%2E4-1985-2.mar)
   (CSA7-2.mar as charmaps/CSA7-2.mar)
   (ISO646-CA2.mar as charmaps/ISO646-CA2.mar)
   (ISO-IR-122.mar as charmaps/ISO-IR-122.mar)
   (CSA_Z243%2E4-1985-GR.mar as charmaps/CSA_Z243%2E4-1985-GR.mar)
   (ISO-IR-123.mar as charmaps/ISO-IR-123.mar)
   (CSN_369103.mar as charmaps/CSN_369103.mar)
   (ISO-IR-139.mar as charmaps/ISO-IR-139.mar)
   (CWI.mar as charmaps/CWI.mar)
   (CP-HU.mar as charmaps/CP-HU.mar)
   (CWI-2.mar as charmaps/CWI-2.mar)
   (DEC-MCS.mar as charmaps/DEC-MCS.mar)
   (DEC.mar as charmaps/DEC.mar)
   (DIN_66003.mar as charmaps/DIN_66003.mar)
   (ISO646-DE.mar as charmaps/ISO646-DE.mar)
   (DE.mar as charmaps/DE.mar)
   (ISO-IR-21.mar as charmaps/ISO-IR-21.mar)
   (DS_2089.mar as charmaps/DS_2089.mar)
   (DK.mar as charmaps/DK.mar)
   (ISO646-DK.mar as charmaps/ISO646-DK.mar)
   (DS2089.mar as charmaps/DS2089.mar)
   (EBCDIC-AT-DE.mar as charmaps/EBCDIC-AT-DE.mar)
   (EBCDIC-AT-DE-A.mar as charmaps/EBCDIC-AT-DE-A.mar)
   (EBCDIC-CA-FR.mar as charmaps/EBCDIC-CA-FR.mar)
   (EBCDIC-DK-NO.mar as charmaps/EBCDIC-DK-NO.mar)
   (EBCDIC-DK-NO-A.mar as charmaps/EBCDIC-DK-NO-A.mar)
   (EBCDIC-ES.mar as charmaps/EBCDIC-ES.mar)
   (EBCDIC-ES-A.mar as charmaps/EBCDIC-ES-A.mar)
   (EBCDIC-ES-S.mar as charmaps/EBCDIC-ES-S.mar)
   (EBCDIC-FI-SE.mar as charmaps/EBCDIC-FI-SE.mar)
   (EBCDIC-FI-SE-A.mar as charmaps/EBCDIC-FI-SE-A.mar)
   (EBCDIC-FR.mar as charmaps/EBCDIC-FR.mar)
   (EBCDIC-IS-FRISS.mar as charmaps/EBCDIC-IS-FRISS.mar)
   (FRISS.mar as charmaps/FRISS.mar)
   (EBCDIC-IT.mar as charmaps/EBCDIC-IT.mar)
   (EBCDIC-PT.mar as charmaps/EBCDIC-PT.mar)
   (EBCDIC-UK.mar as charmaps/EBCDIC-UK.mar)
   (EBCDIC-US.mar as charmaps/EBCDIC-US.mar)
   (ECMA-CYRILLIC.mar as charmaps/ECMA-CYRILLIC.mar)
   (ISO-IR-111.mar as charmaps/ISO-IR-111.mar)
   (ES.mar as charmaps/ES.mar)
   (ISO646-ES.mar as charmaps/ISO646-ES.mar)
   (ISO-IR-17.mar as charmaps/ISO-IR-17.mar)
   (ES2.mar as charmaps/ES2.mar)
   (ISO646-ES2.mar as charmaps/ISO646-ES2.mar)
   (ISO-IR-85.mar as charmaps/ISO-IR-85.mar)
   (EUC-JISX0213.mar as charmaps/EUC-JISX0213.mar)
   (EUC-JP.mar as charmaps/EUC-JP.mar)
   (EUC-JP-MS.mar as charmaps/EUC-JP-MS.mar)
   (EUC-KR.mar as charmaps/EUC-KR.mar)
   (EUC-TW.mar as charmaps/EUC-TW.mar)
   (GB18030.mar as charmaps/GB18030.mar)
   (GB2312.mar as charmaps/GB2312.mar)
   (GBK.mar as charmaps/GBK.mar)
   (WINDOWS-936.mar as charmaps/WINDOWS-936.mar)
   (MS936.mar as charmaps/MS936.mar)
   (CP936.mar as charmaps/CP936.mar)
   (GB_1988-80.mar as charmaps/GB_1988-80.mar)
   (ISO646-CN.mar as charmaps/ISO646-CN.mar)
   (CN.mar as charmaps/CN.mar)
   (ISO-IR-57.mar as charmaps/ISO-IR-57.mar)
   (GEORGIAN-ACADEMY.mar as charmaps/GEORGIAN-ACADEMY.mar)
   (GEORGIAN-PS.mar as charmaps/GEORGIAN-PS.mar)
   (GOST_19768-74.mar as charmaps/GOST_19768-74.mar)
   (ISO-IR-153.mar as charmaps/ISO-IR-153.mar)
   (ST_SEV_358-88.mar as charmaps/ST_SEV_358-88.mar)
   (GREEK-CCITT.mar as charmaps/GREEK-CCITT.mar)
   (ISO-IR-150.mar as charmaps/ISO-IR-150.mar)
   (GREEK7.mar as charmaps/GREEK7.mar)
   (ISO-IR-88.mar as charmaps/ISO-IR-88.mar)
   (GREEK7-OLD.mar as charmaps/GREEK7-OLD.mar)
   (ISO-IR-18.mar as charmaps/ISO-IR-18.mar)
   (HP-GREEK8.mar as charmaps/HP-GREEK8.mar)
   (HP-ROMAN8.mar as charmaps/HP-ROMAN8.mar)
   (R8.mar as charmaps/R8.mar)
   (ROMAN8.mar as charmaps/ROMAN8.mar)
   (HP-ROMAN9.mar as charmaps/HP-ROMAN9.mar)
   (R9.mar as charmaps/R9.mar)
   (ROMAN9.mar as charmaps/ROMAN9.mar)
   (HP-THAI8.mar as charmaps/HP-THAI8.mar)
   (THAI8.mar as charmaps/THAI8.mar)
   (HP-TURKISH8.mar as charmaps/HP-TURKISH8.mar)
   (TURKISH8.mar as charmaps/TURKISH8.mar)
   (IBM037.mar as charmaps/IBM037.mar)
   (EBCDIC-CP-NL.mar as charmaps/EBCDIC-CP-NL.mar)
   (EBCDIC-CP-WT.mar as charmaps/EBCDIC-CP-WT.mar)
   (EBCDIC-CP-CA.mar as charmaps/EBCDIC-CP-CA.mar)
   (EBCDIC-CP-US.mar as charmaps/EBCDIC-CP-US.mar)
   (CP037.mar as charmaps/CP037.mar)
   (IBM038.mar as charmaps/IBM038.mar)
   (CP038.mar as charmaps/CP038.mar)
   (EBCDIC-INT.mar as charmaps/EBCDIC-INT.mar)
   (IBM1004.mar as charmaps/IBM1004.mar)
   (OS2LATIN1.mar as charmaps/OS2LATIN1.mar)
   (CP1004.mar as charmaps/CP1004.mar)
   (IBM1026.mar as charmaps/IBM1026.mar)
   (1026.mar as charmaps/1026.mar)
   (CP1026.mar as charmaps/CP1026.mar)
   (IBM1047.mar as charmaps/IBM1047.mar)
   (1047.mar as charmaps/1047.mar)
   (CP1047.mar as charmaps/CP1047.mar)
   (IBM-1047.mar as charmaps/IBM-1047.mar)
   (IBM1124.mar as charmaps/IBM1124.mar)
   (CP1124.mar as charmaps/CP1124.mar)
   (IBM1129.mar as charmaps/IBM1129.mar)
   (CP1129.mar as charmaps/CP1129.mar)
   (IBM1132.mar as charmaps/IBM1132.mar)
   (CP1132.mar as charmaps/CP1132.mar)
   (IBM1133.mar as charmaps/IBM1133.mar)
   (IBM1162.mar as charmaps/IBM1162.mar)
   (CP1133.mar as charmaps/CP1133.mar)
   (IBM1160.mar as charmaps/IBM1160.mar)
   (CP1160.mar as charmaps/CP1160.mar)
   (IBM1161.mar as charmaps/IBM1161.mar)
   (CP1161.mar as charmaps/CP1161.mar)
   (IBM1163.mar as charmaps/IBM1163.mar)
   (CP1163.mar as charmaps/CP1163.mar)
   (IBM1164.mar as charmaps/IBM1164.mar)
   (CP1164.mar as charmaps/CP1164.mar)
   (IBM256.mar as charmaps/IBM256.mar)
   (EBCDIC-INT1.mar as charmaps/EBCDIC-INT1.mar)
   (IBM273.mar as charmaps/IBM273.mar)
   (CP273.mar as charmaps/CP273.mar)
   (IBM274.mar as charmaps/IBM274.mar)
   (CP274.mar as charmaps/CP274.mar)
   (EBCDIC-BE.mar as charmaps/EBCDIC-BE.mar)
   (IBM275.mar as charmaps/IBM275.mar)
   (CP275.mar as charmaps/CP275.mar)
   (EBCDIC-BR.mar as charmaps/EBCDIC-BR.mar)
   (IBM277.mar as charmaps/IBM277.mar)
   (EBCDIC-CP-NO.mar as charmaps/EBCDIC-CP-NO.mar)
   (EBCDIC-CP-DK.mar as charmaps/EBCDIC-CP-DK.mar)
   (IBM278.mar as charmaps/IBM278.mar)
   (EBCDIC-CP-SE.mar as charmaps/EBCDIC-CP-SE.mar)
   (EBCDIC-CP-FI.mar as charmaps/EBCDIC-CP-FI.mar)
   (CP278.mar as charmaps/CP278.mar)
   (IBM280.mar as charmaps/IBM280.mar)
   (EBCDIC-CP-IT.mar as charmaps/EBCDIC-CP-IT.mar)
   (CP280.mar as charmaps/CP280.mar)
   (IBM281.mar as charmaps/IBM281.mar)
   (CP281.mar as charmaps/CP281.mar)
   (EBCDIC-JP-E.mar as charmaps/EBCDIC-JP-E.mar)
   (IBM284.mar as charmaps/IBM284.mar)
   (EBCDIC-CP-ES.mar as charmaps/EBCDIC-CP-ES.mar)
   (CP284.mar as charmaps/CP284.mar)
   (IBM285.mar as charmaps/IBM285.mar)
   (EBCDIC-CP-GB.mar as charmaps/EBCDIC-CP-GB.mar)
   (CP285.mar as charmaps/CP285.mar)
   (IBM290.mar as charmaps/IBM290.mar)
   (EBCDIC-JP-KANA.mar as charmaps/EBCDIC-JP-KANA.mar)
   (CP290.mar as charmaps/CP290.mar)
   (IBM297.mar as charmaps/IBM297.mar)
   (EBCDIC-CP-FR.mar as charmaps/EBCDIC-CP-FR.mar)
   (CP297.mar as charmaps/CP297.mar)
   (IBM420.mar as charmaps/IBM420.mar)
   (EBCDIC-CP-AR1.mar as charmaps/EBCDIC-CP-AR1.mar)
   (CP420.mar as charmaps/CP420.mar)
   (IBM423.mar as charmaps/IBM423.mar)
   (EBCDIC-CP-GR.mar as charmaps/EBCDIC-CP-GR.mar)
   (CP423.mar as charmaps/CP423.mar)
   (IBM424.mar as charmaps/IBM424.mar)
   (EBCDIC-CP-HE.mar as charmaps/EBCDIC-CP-HE.mar)
   (CP424.mar as charmaps/CP424.mar)
   (IBM437.mar as charmaps/IBM437.mar)
   (437.mar as charmaps/437.mar)
   (CP437.mar as charmaps/CP437.mar)
   (IBM500.mar as charmaps/IBM500.mar)
   (EBCDIC-CP-CH.mar as charmaps/EBCDIC-CP-CH.mar)
   (EBCDIC-CP-BE.mar as charmaps/EBCDIC-CP-BE.mar)
   (500V1.mar as charmaps/500V1.mar)
   (500.mar as charmaps/500.mar)
   (CP500.mar as charmaps/CP500.mar)
   (IBM850.mar as charmaps/IBM850.mar)
   (850.mar as charmaps/850.mar)
   (CP850.mar as charmaps/CP850.mar)
   (IBM851.mar as charmaps/IBM851.mar)
   (851.mar as charmaps/851.mar)
   (CP851.mar as charmaps/CP851.mar)
   (IBM852.mar as charmaps/IBM852.mar)
   (852.mar as charmaps/852.mar)
   (CP852.mar as charmaps/CP852.mar)
   (IBM855.mar as charmaps/IBM855.mar)
   (855.mar as charmaps/855.mar)
   (CP855.mar as charmaps/CP855.mar)
   (IBM856.mar as charmaps/IBM856.mar)
   (856.mar as charmaps/856.mar)
   (CP856.mar as charmaps/CP856.mar)
   (IBM857.mar as charmaps/IBM857.mar)
   (857.mar as charmaps/857.mar)
   (CP857.mar as charmaps/CP857.mar)
   (IBM860.mar as charmaps/IBM860.mar)
   (860.mar as charmaps/860.mar)
   (CP860.mar as charmaps/CP860.mar)
   (IBM861.mar as charmaps/IBM861.mar)
   (CP-IS.mar as charmaps/CP-IS.mar)
   (861.mar as charmaps/861.mar)
   (CP861.mar as charmaps/CP861.mar)
   (IBM862.mar as charmaps/IBM862.mar)
   (862.mar as charmaps/862.mar)
   (CP862.mar as charmaps/CP862.mar)
   (IBM863.mar as charmaps/IBM863.mar)
   (863.mar as charmaps/863.mar)
   (CP863.mar as charmaps/CP863.mar)
   (IBM864.mar as charmaps/IBM864.mar)
   (CP864.mar as charmaps/CP864.mar)
   (IBM865.mar as charmaps/IBM865.mar)
   (865.mar as charmaps/865.mar)
   (CP865.mar as charmaps/CP865.mar)
   (IBM866.mar as charmaps/IBM866.mar)
   (866.mar as charmaps/866.mar)
   (CP866.mar as charmaps/CP866.mar)
   (IBM866NAV.mar as charmaps/IBM866NAV.mar)
   (866NAV.mar as charmaps/866NAV.mar)
   (CP866NAV.mar as charmaps/CP866NAV.mar)
   (IBM868.mar as charmaps/IBM868.mar)
   (CP-AR.mar as charmaps/CP-AR.mar)
   (CP868.mar as charmaps/CP868.mar)
   (IBM869.mar as charmaps/IBM869.mar)
   (CP-GR.mar as charmaps/CP-GR.mar)
   (869.mar as charmaps/869.mar)
   (CP869.mar as charmaps/CP869.mar)
   (IBM870.mar as charmaps/IBM870.mar)
   (EBCDIC-CP-YU.mar as charmaps/EBCDIC-CP-YU.mar)
   (EBCDIC-CP-ROECE.mar as charmaps/EBCDIC-CP-ROECE.mar)
   (CP870.mar as charmaps/CP870.mar)
   (IBM871.mar as charmaps/IBM871.mar)
   (EBCDIC-CP-IS.mar as charmaps/EBCDIC-CP-IS.mar)
   (CP871.mar as charmaps/CP871.mar)
   (IBM874.mar as charmaps/IBM874.mar)
   (WINDOWS-874.mar as charmaps/WINDOWS-874.mar)
   (CP874.mar as charmaps/CP874.mar)
   (IBM875.mar as charmaps/IBM875.mar)
   (EBCDIC-GREEK.mar as charmaps/EBCDIC-GREEK.mar)
   (CP875.mar as charmaps/CP875.mar)
   (IBM880.mar as charmaps/IBM880.mar)
   (EBCDIC-CYRILLIC.mar as charmaps/EBCDIC-CYRILLIC.mar)
   (CP880.mar as charmaps/CP880.mar)
   (IBM891.mar as charmaps/IBM891.mar)
   (CP891.mar as charmaps/CP891.mar)
   (IBM903.mar as charmaps/IBM903.mar)
   (CP903.mar as charmaps/CP903.mar)
   (IBM904.mar as charmaps/IBM904.mar)
   (904.mar as charmaps/904.mar)
   (CP904.mar as charmaps/CP904.mar)
   (IBM905.mar as charmaps/IBM905.mar)
   (EBCDIC-CP-TR.mar as charmaps/EBCDIC-CP-TR.mar)
   (CP905.mar as charmaps/CP905.mar)
   (IBM918.mar as charmaps/IBM918.mar)
   (EBCDIC-CP-AR2.mar as charmaps/EBCDIC-CP-AR2.mar)
   (CP918.mar as charmaps/CP918.mar)
   (IBM922.mar as charmaps/IBM922.mar)
   (CP922.mar as charmaps/CP922.mar)
   (IEC_P27-1.mar as charmaps/IEC_P27-1.mar)
   (ISO-IR-143.mar as charmaps/ISO-IR-143.mar)
   (INIS.mar as charmaps/INIS.mar)
   (ISO-IR-49.mar as charmaps/ISO-IR-49.mar)
   (INIS-8.mar as charmaps/INIS-8.mar)
   (ISO-IR-50.mar as charmaps/ISO-IR-50.mar)
   (INIS-CYRILLIC.mar as charmaps/INIS-CYRILLIC.mar)
   (ISO-IR-51.mar as charmaps/ISO-IR-51.mar)
   (INVARIANT.mar as charmaps/INVARIANT.mar)
   (ISO-IR-170.mar as charmaps/ISO-IR-170.mar)
   (ISIRI-3342.mar as charmaps/ISIRI-3342.mar)
   (ISO-8859-1.mar as charmaps/ISO-8859-1.mar)
   (CP819.mar as charmaps/CP819.mar)
   (IBM819.mar as charmaps/IBM819.mar)
   (L1.mar as charmaps/L1.mar)
   (LATIN1.mar as charmaps/LATIN1.mar)
   (ISO_8859-1.mar as charmaps/ISO_8859-1.mar)
   (ISO_8859-1%3A1987.mar as charmaps/ISO_8859-1%3A1987.mar)
   (ISO-IR-100.mar as charmaps/ISO-IR-100.mar)
   (ISO-8859-10.mar as charmaps/ISO-8859-10.mar)
   (LATIN6.mar as charmaps/LATIN6.mar)
   (L6.mar as charmaps/L6.mar)
   (ISO_8859-10.mar as charmaps/ISO_8859-10.mar)
   (ISO_8859-10%3A1992.mar as charmaps/ISO_8859-10%3A1992.mar)
   (ISO-IR-157.mar as charmaps/ISO-IR-157.mar)
   (ISO-8859-11.mar as charmaps/ISO-8859-11.mar)
   (ISO-8859-13.mar as charmaps/ISO-8859-13.mar)
   (L7.mar as charmaps/L7.mar)
   (LATIN7.mar as charmaps/LATIN7.mar)
   (ISO-IR-179.mar as charmaps/ISO-IR-179.mar)
   (ISO-8859-14.mar as charmaps/ISO-8859-14.mar)
   (L8.mar as charmaps/L8.mar)
   (LATIN8.mar as charmaps/LATIN8.mar)
   (ISO-8859-15.mar as charmaps/ISO-8859-15.mar)
   (LATIN-9.mar as charmaps/LATIN-9.mar)
   (ISO_8859-15.mar as charmaps/ISO_8859-15.mar)
   (ISO-8859-16.mar as charmaps/ISO-8859-16.mar)
   (L10.mar as charmaps/L10.mar)
   (LATIN10.mar as charmaps/LATIN10.mar)
   (ISO_8859-16.mar as charmaps/ISO_8859-16.mar)
   (ISO_8859-16%3A2001.mar as charmaps/ISO_8859-16%3A2001.mar)
   (ISO-IR-226.mar as charmaps/ISO-IR-226.mar)
   (ISO-8859-2.mar as charmaps/ISO-8859-2.mar)
   (L2.mar as charmaps/L2.mar)
   (LATIN2.mar as charmaps/LATIN2.mar)
   (ISO_8859-2.mar as charmaps/ISO_8859-2.mar)
   (ISO_8859-2%3A1987.mar as charmaps/ISO_8859-2%3A1987.mar)
   (ISO-IR-101.mar as charmaps/ISO-IR-101.mar)
   (ISO-8859-3.mar as charmaps/ISO-8859-3.mar)
   (L3.mar as charmaps/L3.mar)
   (LATIN3.mar as charmaps/LATIN3.mar)
   (ISO_8859-3.mar as charmaps/ISO_8859-3.mar)
   (ISO_8859-3%3A1988.mar as charmaps/ISO_8859-3%3A1988.mar)
   (ISO-IR-109.mar as charmaps/ISO-IR-109.mar)
   (ISO-8859-4.mar as charmaps/ISO-8859-4.mar)
   (L4.mar as charmaps/L4.mar)
   (LATIN4.mar as charmaps/LATIN4.mar)
   (ISO_8859-4.mar as charmaps/ISO_8859-4.mar)
   (ISO_8859-4%3A1988.mar as charmaps/ISO_8859-4%3A1988.mar)
   (ISO-IR-110.mar as charmaps/ISO-IR-110.mar)
   (ISO-8859-5.mar as charmaps/ISO-8859-5.mar)
   (CYRILLIC.mar as charmaps/CYRILLIC.mar)
   (ISO_8859-5.mar as charmaps/ISO_8859-5.mar)
   (ISO_8859-5%3A1988.mar as charmaps/ISO_8859-5%3A1988.mar)
   (ISO-IR-144.mar as charmaps/ISO-IR-144.mar)
   (ISO-8859-6.mar as charmaps/ISO-8859-6.mar)
   (ARABIC.mar as charmaps/ARABIC.mar)
   (ASMO-708.mar as charmaps/ASMO-708.mar)
   (ECMA-114.mar as charmaps/ECMA-114.mar)
   (ISO_8859-6.mar as charmaps/ISO_8859-6.mar)
   (ISO_8859-6%3A1987.mar as charmaps/ISO_8859-6%3A1987.mar)
   (ISO-IR-127.mar as charmaps/ISO-IR-127.mar)
   (ISO-8859-7.mar as charmaps/ISO-8859-7.mar)
   (GREEK8.mar as charmaps/GREEK8.mar)
   (GREEK.mar as charmaps/GREEK.mar)
   (ECMA-118.mar as charmaps/ECMA-118.mar)
   (ELOT_928.mar as charmaps/ELOT_928.mar)
   (ISO_8859-7.mar as charmaps/ISO_8859-7.mar)
   (ISO_8859-7%3A1987.mar as charmaps/ISO_8859-7%3A1987.mar)
   (ISO_8859-7%3A2003.mar as charmaps/ISO_8859-7%3A2003.mar)
   (ISO-IR-126.mar as charmaps/ISO-IR-126.mar)
   (ISO-8859-8.mar as charmaps/ISO-8859-8.mar)
   (HEBREW.mar as charmaps/HEBREW.mar)
   (ISO_8859-8.mar as charmaps/ISO_8859-8.mar)
   (ISO_8859-8%3A1988.mar as charmaps/ISO_8859-8%3A1988.mar)
   (ISO-IR-138.mar as charmaps/ISO-IR-138.mar)
   (ISO-8859-9.mar as charmaps/ISO-8859-9.mar)
   (L5.mar as charmaps/L5.mar)
   (LATIN5.mar as charmaps/LATIN5.mar)
   (ISO_8859-9.mar as charmaps/ISO_8859-9.mar)
   (ISO_8859-9%3A1989.mar as charmaps/ISO_8859-9%3A1989.mar)
   (ISO-IR-148.mar as charmaps/ISO-IR-148.mar)
   (ISO-8859-9E.mar as charmaps/ISO-8859-9E.mar)
   (ISO-IR-197.mar as charmaps/ISO-IR-197.mar)
   (ISO-IR-209.mar as charmaps/ISO-IR-209.mar)
   (ISO-IR-90.mar as charmaps/ISO-IR-90.mar)
   (ISO_6937-2%3A1983.mar as charmaps/ISO_6937-2%3A1983.mar)
   (ISO_10367-BOX.mar as charmaps/ISO_10367-BOX.mar)
   (ISO-IR-155.mar as charmaps/ISO-IR-155.mar)
   (ISO_11548-1.mar as charmaps/ISO_11548-1.mar)
   (ISO%2FTR_11548-1.mar as charmaps/ISO%2FTR_11548-1.mar)
   (ISO_2033-1983.mar as charmaps/ISO_2033-1983.mar)
   (E13B.mar as charmaps/E13B.mar)
   (ISO-IR-98.mar as charmaps/ISO-IR-98.mar)
   (ISO_5427.mar as charmaps/ISO_5427.mar)
   (KOI-7.mar as charmaps/KOI-7.mar)
   (ISO-IR-37.mar as charmaps/ISO-IR-37.mar)
   (ISO_5427-EXT.mar as charmaps/ISO_5427-EXT.mar)
   (ISO_5427%3A1981.mar as charmaps/ISO_5427%3A1981.mar)
   (ISO-IR-54.mar as charmaps/ISO-IR-54.mar)
   (ISO_5428.mar as charmaps/ISO_5428.mar)
   (ISO_5428%3A1980.mar as charmaps/ISO_5428%3A1980.mar)
   (ISO-IR-55.mar as charmaps/ISO-IR-55.mar)
   (ISO_646%2EBASIC.mar as charmaps/ISO_646%2EBASIC.mar)
   (ISO_646%2EBASIC%3A1983.mar as charmaps/ISO_646%2EBASIC%3A1983.mar)
   (REF.mar as charmaps/REF.mar)
   (ISO_646%2EIRV.mar as charmaps/ISO_646%2EIRV.mar)
   (IRV.mar as charmaps/IRV.mar)
   (ISO_646%2EIRV%3A1983.mar as charmaps/ISO_646%2EIRV%3A1983.mar)
   (ISO-IR-2.mar as charmaps/ISO-IR-2.mar)
   (ISO_6937.mar as charmaps/ISO_6937.mar)
   (ISO6937.mar as charmaps/ISO6937.mar)
   (ISO_6937%3A1992.mar as charmaps/ISO_6937%3A1992.mar)
   (ISO-IR-156.mar as charmaps/ISO-IR-156.mar)
   (ISO_6937-2-25.mar as charmaps/ISO_6937-2-25.mar)
   (ISO-IR-152.mar as charmaps/ISO-IR-152.mar)
   (ISO_6937-2-ADD.mar as charmaps/ISO_6937-2-ADD.mar)
   (ISO-IR-142.mar as charmaps/ISO-IR-142.mar)
   (ISO_8859-SUPP.mar as charmaps/ISO_8859-SUPP.mar)
   (LATIN1-2-5.mar as charmaps/LATIN1-2-5.mar)
   (ISO-IR-154.mar as charmaps/ISO-IR-154.mar)
   (IT.mar as charmaps/IT.mar)
   (ISO646-IT.mar as charmaps/ISO646-IT.mar)
   (ISO-IR-15.mar as charmaps/ISO-IR-15.mar)
   (JIS_C6220-1969-RO.mar as charmaps/JIS_C6220-1969-RO.mar)
   (ISO646-JP.mar as charmaps/ISO646-JP.mar)
   (JP.mar as charmaps/JP.mar)
   (ISO-IR-14.mar as charmaps/ISO-IR-14.mar)
   (JIS_C6229-1984-B.mar as charmaps/JIS_C6229-1984-B.mar)
   (JP-OCR-B.mar as charmaps/JP-OCR-B.mar)
   (ISO646-JP-OCR-B.mar as charmaps/ISO646-JP-OCR-B.mar)
   (ISO-IR-92.mar as charmaps/ISO-IR-92.mar)
   (JIS_X0201.mar as charmaps/JIS_X0201.mar)
   (X0201.mar as charmaps/X0201.mar)
   (JOHAB.mar as charmaps/JOHAB.mar)
   (JUS_I%2EB1%2E002.mar as charmaps/JUS_I%2EB1%2E002.mar)
   (YU.mar as charmaps/YU.mar)
   (JS.mar as charmaps/JS.mar)
   (ISO646-YU.mar as charmaps/ISO646-YU.mar)
   (ISO-IR-141.mar as charmaps/ISO-IR-141.mar)
   (JUS_I%2EB1%2E003-MAC.mar as charmaps/JUS_I%2EB1%2E003-MAC.mar)
   (ISO-IR-147.mar as charmaps/ISO-IR-147.mar)
   (MACEDONIAN.mar as charmaps/MACEDONIAN.mar)
   (JUS_I%2EB1%2E003-SERB.mar as charmaps/JUS_I%2EB1%2E003-SERB.mar)
   (SERBIAN.mar as charmaps/SERBIAN.mar)
   (ISO-IR-146.mar as charmaps/ISO-IR-146.mar)
   (KOI-8.mar as charmaps/KOI-8.mar)
   (KOI8-R.mar as charmaps/KOI8-R.mar)
   (KOI8-RU.mar as charmaps/KOI8-RU.mar)
   (KOI8-T.mar as charmaps/KOI8-T.mar)
   (KOI8-U.mar as charmaps/KOI8-U.mar)
   (KSC5636.mar as charmaps/KSC5636.mar)
   (ISO646-KR.mar as charmaps/ISO646-KR.mar)
   (LATIN-GREEK.mar as charmaps/LATIN-GREEK.mar)
   (ISO-IR-19.mar as charmaps/ISO-IR-19.mar)
   (LATIN-GREEK-1.mar as charmaps/LATIN-GREEK-1.mar)
   (ISO-IR-27.mar as charmaps/ISO-IR-27.mar)
   (MAC-CYRILLIC.mar as charmaps/MAC-CYRILLIC.mar)
   (MAC-IS.mar as charmaps/MAC-IS.mar)
   (MAC-SAMI.mar as charmaps/MAC-SAMI.mar)
   (MAC-UK.mar as charmaps/MAC-UK.mar)
   (MACINTOSH.mar as charmaps/MACINTOSH.mar)
   (MAC.mar as charmaps/MAC.mar)
   (MIK.mar as charmaps/MIK.mar)
   (MSZ_7795%2E3.mar as charmaps/MSZ_7795%2E3.mar)
   (HU.mar as charmaps/HU.mar)
   (ISO646-HU.mar as charmaps/ISO646-HU.mar)
   (ISO-IR-86.mar as charmaps/ISO-IR-86.mar)
   (NATS-DANO.mar as charmaps/NATS-DANO.mar)
   (ISO-IR-9-1.mar as charmaps/ISO-IR-9-1.mar)
   (NATS-SEFI.mar as charmaps/NATS-SEFI.mar)
   (ISO-IR-8-1.mar as charmaps/ISO-IR-8-1.mar)
   (NC_NC00-10.mar as charmaps/NC_NC00-10.mar)
   (ISO646-CU.mar as charmaps/ISO646-CU.mar)
   (ISO-IR-151.mar as charmaps/ISO-IR-151.mar)
   (NC_NC00-10%3A81.mar as charmaps/NC_NC00-10%3A81.mar)
   (CUBA.mar as charmaps/CUBA.mar)
   (NEXTSTEP.mar as charmaps/NEXTSTEP.mar)
   (NEXT.mar as charmaps/NEXT.mar)
   (NF_Z_62-010.mar as charmaps/NF_Z_62-010.mar)
   (FR.mar as charmaps/FR.mar)
   (ISO646-FR.mar as charmaps/ISO646-FR.mar)
   (ISO-IR-69.mar as charmaps/ISO-IR-69.mar)
   (NF_Z_62-010_%281973%29.mar as charmaps/NF_Z_62-010_%281973%29.mar)
   (ISO646-FR1.mar as charmaps/ISO646-FR1.mar)
   (ISO-IR-25.mar as charmaps/ISO-IR-25.mar)
   (NS_4551-1.mar as charmaps/NS_4551-1.mar)
   (NO.mar as charmaps/NO.mar)
   (ISO646-NO.mar as charmaps/ISO646-NO.mar)
   (ISO-IR-60.mar as charmaps/ISO-IR-60.mar)
   (NS_4551-2.mar as charmaps/NS_4551-2.mar)
   (NO2.mar as charmaps/NO2.mar)
   (ISO-IR-61.mar as charmaps/ISO-IR-61.mar)
   (ISO646-NO2.mar as charmaps/ISO646-NO2.mar)
   (PT.mar as charmaps/PT.mar)
   (ISO646-PT.mar as charmaps/ISO646-PT.mar)
   (ISO-IR-16.mar as charmaps/ISO-IR-16.mar)
   (PT154.mar as charmaps/PT154.mar)
   (Cyrillic-Asian.mar as charmaps/Cyrillic-Asian.mar)
   (PT2.mar as charmaps/PT2.mar)
   (ISO646-PT2.mar as charmaps/ISO646-PT2.mar)
   (ISO-IR-84.mar as charmaps/ISO-IR-84.mar)
   (RK1048.mar as charmaps/RK1048.mar)
   (STRK1048-2002.mar as charmaps/STRK1048-2002.mar)
   (SAMI.mar as charmaps/SAMI.mar)
   (ISO-IR-158.mar as charmaps/ISO-IR-158.mar)
   (LAP.mar as charmaps/LAP.mar)
   (LATIN-LAP.mar as charmaps/LATIN-LAP.mar)
   (WIN-SAMI-2.mar as charmaps/WIN-SAMI-2.mar)
   (WINDOWS-SAMI2.mar as charmaps/WINDOWS-SAMI2.mar)
   (WS2.mar as charmaps/WS2.mar)
   (SEN_850200_B.mar as charmaps/SEN_850200_B.mar)
   (SS636127.mar as charmaps/SS636127.mar)
   (SE.mar as charmaps/SE.mar)
   (ISO646-SE.mar as charmaps/ISO646-SE.mar)
   (ISO646-FI.mar as charmaps/ISO646-FI.mar)
   (FI.mar as charmaps/FI.mar)
   (ISO-IR-10.mar as charmaps/ISO-IR-10.mar)
   (SEN_850200_C.mar as charmaps/SEN_850200_C.mar)
   (SE2.mar as charmaps/SE2.mar)
   (ISO646-SE2.mar as charmaps/ISO646-SE2.mar)
   (ISO-IR-11.mar as charmaps/ISO-IR-11.mar)
   (SHIFT_JIS.mar as charmaps/SHIFT_JIS.mar)
   (SJIS.mar as charmaps/SJIS.mar)
   (SHIFT_JISX0213.mar as charmaps/SHIFT_JISX0213.mar)
   (T%2E101-G2.mar as charmaps/T%2E101-G2.mar)
   (ISO-IR-128.mar as charmaps/ISO-IR-128.mar)
   (T%2E61-7BIT.mar as charmaps/T%2E61-7BIT.mar)
   (ISO-IR-102.mar as charmaps/ISO-IR-102.mar)
   (T%2E61-8BIT.mar as charmaps/T%2E61-8BIT.mar)
   (ISO-IR-103.mar as charmaps/ISO-IR-103.mar)
   (T%2E61.mar as charmaps/T%2E61.mar)
   (TCVN5712-1.mar as charmaps/TCVN5712-1.mar)
   (TCVN5712-1%3A1993.mar as charmaps/TCVN5712-1%3A1993.mar)
   (TCVN-5712.mar as charmaps/TCVN-5712.mar)
   (TCVN.mar as charmaps/TCVN.mar)
   (TIS-620.mar as charmaps/TIS-620.mar)
   (ISO-IR-166.mar as charmaps/ISO-IR-166.mar)
   (TIS620%2E2533-0.mar as charmaps/TIS620%2E2533-0.mar)
   (TIS620%2E2529-1.mar as charmaps/TIS620%2E2529-1.mar)
   (TIS620-0.mar as charmaps/TIS620-0.mar)
   (TIS620.mar as charmaps/TIS620.mar)
   (UTF-8.mar as charmaps/UTF-8.mar)
   (ISO-10646%2FUTF-8.mar as charmaps/ISO-10646%2FUTF-8.mar)
   (VIDEOTEX-SUPPL.mar as charmaps/VIDEOTEX-SUPPL.mar)
   (ISO-IR-70.mar as charmaps/ISO-IR-70.mar)
   (VISCII.mar as charmaps/VISCII.mar)
   (WINDOWS-31J.mar as charmaps/WINDOWS-31J.mar)
    ))

(rule
 (targets ANSI_X3%2E110-1983.mar NAPLPS.mar CSA_T500-1983.mar ISO-IR-99.mar)
 (deps    ANSI_X3.110-1983)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ANSI_X3%2E4-1968.mar CP367.mar IBM367.mar US.mar US-ASCII.mar ISO646-US.mar ASCII.mar ISO_646%2EIRV%3A1991.mar ANSI_X3%2E4-1986.mar ISO-IR-6.mar)
 (deps    ANSI_X3.4-1968)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ARMSCII-8.mar)
 (deps    ARMSCII-8)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ASMO_449.mar ISO-IR-89.mar ARABIC7.mar ISO_9036.mar)
 (deps    ASMO_449)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets BIG5.mar BIG5-CP950.mar)
 (deps    BIG5)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets BIG5-HKSCS.mar BIG5HKSCS.mar)
 (deps    BIG5-HKSCS)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets BRF.mar)
 (deps    BRF)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets BS_4730.mar UK.mar GB.mar ISO646-GB.mar ISO-IR-4.mar)
 (deps    BS_4730)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets BS_VIEWDATA.mar ISO-IR-47.mar)
 (deps    BS_VIEWDATA)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets CP10007.mar)
 (deps    CP10007)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets CP1125.mar IBM848.mar RUSCII.mar)
 (deps    CP1125)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets CP1250.mar MS-EE.mar)
 (deps    CP1250)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets CP1251.mar MS-CYRL.mar)
 (deps    CP1251)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets CP1252.mar MS-ANSI.mar)
 (deps    CP1252)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets CP1253.mar MS-GREEK.mar)
 (deps    CP1253)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets CP1254.mar MS-TURK.mar)
 (deps    CP1254)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets CP1255.mar MS-HEBR.mar)
 (deps    CP1255)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets CP1256.mar MS-ARAB.mar)
 (deps    CP1256)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets CP1257.mar WINBALTRIM.mar)
 (deps    CP1257)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets CP1258.mar)
 (deps    CP1258)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets CP737.mar)
 (deps    CP737)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets CP775.mar)
 (deps    CP775)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets CP932.mar)
 (deps    CP932)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets CP949.mar)
 (deps    CP949)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets CSA_Z243%2E4-1985-1.mar CA.mar CSA7-1.mar ISO646-CA.mar ISO-IR-121.mar)
 (deps    CSA_Z243.4-1985-1)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets CSA_Z243%2E4-1985-2.mar CSA7-2.mar ISO646-CA2.mar ISO-IR-122.mar)
 (deps    CSA_Z243.4-1985-2)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets CSA_Z243%2E4-1985-GR.mar ISO-IR-123.mar)
 (deps    CSA_Z243.4-1985-GR)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets CSN_369103.mar ISO-IR-139.mar)
 (deps    CSN_369103)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets CWI.mar CP-HU.mar CWI-2.mar)
 (deps    CWI)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets DEC-MCS.mar DEC.mar)
 (deps    DEC-MCS)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets DIN_66003.mar ISO646-DE.mar DE.mar ISO-IR-21.mar)
 (deps    DIN_66003)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets DS_2089.mar DK.mar ISO646-DK.mar DS2089.mar)
 (deps    DS_2089)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets EBCDIC-AT-DE.mar)
 (deps    EBCDIC-AT-DE)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets EBCDIC-AT-DE-A.mar)
 (deps    EBCDIC-AT-DE-A)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets EBCDIC-CA-FR.mar)
 (deps    EBCDIC-CA-FR)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets EBCDIC-DK-NO.mar)
 (deps    EBCDIC-DK-NO)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets EBCDIC-DK-NO-A.mar)
 (deps    EBCDIC-DK-NO-A)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets EBCDIC-ES.mar)
 (deps    EBCDIC-ES)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets EBCDIC-ES-A.mar)
 (deps    EBCDIC-ES-A)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets EBCDIC-ES-S.mar)
 (deps    EBCDIC-ES-S)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets EBCDIC-FI-SE.mar)
 (deps    EBCDIC-FI-SE)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets EBCDIC-FI-SE-A.mar)
 (deps    EBCDIC-FI-SE-A)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets EBCDIC-FR.mar)
 (deps    EBCDIC-FR)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets EBCDIC-IS-FRISS.mar FRISS.mar)
 (deps    EBCDIC-IS-FRISS)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets EBCDIC-IT.mar)
 (deps    EBCDIC-IT)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets EBCDIC-PT.mar)
 (deps    EBCDIC-PT)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets EBCDIC-UK.mar)
 (deps    EBCDIC-UK)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets EBCDIC-US.mar)
 (deps    EBCDIC-US)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ECMA-CYRILLIC.mar ISO-IR-111.mar)
 (deps    ECMA-CYRILLIC)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ES.mar ISO646-ES.mar ISO-IR-17.mar)
 (deps    ES)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ES2.mar ISO646-ES2.mar ISO-IR-85.mar)
 (deps    ES2)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets EUC-JISX0213.mar)
 (deps    EUC-JISX0213)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets EUC-JP.mar)
 (deps    EUC-JP)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets EUC-JP-MS.mar)
 (deps    EUC-JP-MS)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets EUC-KR.mar)
 (deps    EUC-KR)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets EUC-TW.mar)
 (deps    EUC-TW)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets GB18030.mar)
 (deps    GB18030)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets GB2312.mar)
 (deps    GB2312)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets GBK.mar WINDOWS-936.mar MS936.mar CP936.mar)
 (deps    GBK)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets GB_1988-80.mar ISO646-CN.mar CN.mar ISO-IR-57.mar)
 (deps    GB_1988-80)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets GEORGIAN-ACADEMY.mar)
 (deps    GEORGIAN-ACADEMY)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets GEORGIAN-PS.mar)
 (deps    GEORGIAN-PS)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets GOST_19768-74.mar ISO-IR-153.mar ST_SEV_358-88.mar)
 (deps    GOST_19768-74)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets GREEK-CCITT.mar ISO-IR-150.mar)
 (deps    GREEK-CCITT)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets GREEK7.mar ISO-IR-88.mar)
 (deps    GREEK7)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets GREEK7-OLD.mar ISO-IR-18.mar)
 (deps    GREEK7-OLD)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets HP-GREEK8.mar)
 (deps    HP-GREEK8)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets HP-ROMAN8.mar R8.mar ROMAN8.mar)
 (deps    HP-ROMAN8)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets HP-ROMAN9.mar R9.mar ROMAN9.mar)
 (deps    HP-ROMAN9)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets HP-THAI8.mar THAI8.mar)
 (deps    HP-THAI8)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets HP-TURKISH8.mar TURKISH8.mar)
 (deps    HP-TURKISH8)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM037.mar EBCDIC-CP-NL.mar EBCDIC-CP-WT.mar EBCDIC-CP-CA.mar EBCDIC-CP-US.mar CP037.mar)
 (deps    IBM037)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM038.mar CP038.mar EBCDIC-INT.mar)
 (deps    IBM038)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM1004.mar OS2LATIN1.mar CP1004.mar)
 (deps    IBM1004)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM1026.mar 1026.mar CP1026.mar)
 (deps    IBM1026)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM1047.mar 1047.mar CP1047.mar IBM-1047.mar)
 (deps    IBM1047)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM1124.mar CP1124.mar)
 (deps    IBM1124)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM1129.mar CP1129.mar)
 (deps    IBM1129)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM1132.mar CP1132.mar)
 (deps    IBM1132)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM1133.mar IBM1162.mar CP1133.mar)
 (deps    IBM1133)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM1160.mar CP1160.mar)
 (deps    IBM1160)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM1161.mar CP1161.mar)
 (deps    IBM1161)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM1163.mar CP1163.mar)
 (deps    IBM1163)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM1164.mar CP1164.mar)
 (deps    IBM1164)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM256.mar EBCDIC-INT1.mar)
 (deps    IBM256)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM273.mar CP273.mar)
 (deps    IBM273)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM274.mar CP274.mar EBCDIC-BE.mar)
 (deps    IBM274)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM275.mar CP275.mar EBCDIC-BR.mar)
 (deps    IBM275)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM277.mar EBCDIC-CP-NO.mar EBCDIC-CP-DK.mar)
 (deps    IBM277)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM278.mar EBCDIC-CP-SE.mar EBCDIC-CP-FI.mar CP278.mar)
 (deps    IBM278)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM280.mar EBCDIC-CP-IT.mar CP280.mar)
 (deps    IBM280)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM281.mar CP281.mar EBCDIC-JP-E.mar)
 (deps    IBM281)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM284.mar EBCDIC-CP-ES.mar CP284.mar)
 (deps    IBM284)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM285.mar EBCDIC-CP-GB.mar CP285.mar)
 (deps    IBM285)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM290.mar EBCDIC-JP-KANA.mar CP290.mar)
 (deps    IBM290)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM297.mar EBCDIC-CP-FR.mar CP297.mar)
 (deps    IBM297)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM420.mar EBCDIC-CP-AR1.mar CP420.mar)
 (deps    IBM420)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM423.mar EBCDIC-CP-GR.mar CP423.mar)
 (deps    IBM423)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM424.mar EBCDIC-CP-HE.mar CP424.mar)
 (deps    IBM424)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM437.mar 437.mar CP437.mar)
 (deps    IBM437)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM500.mar EBCDIC-CP-CH.mar EBCDIC-CP-BE.mar 500V1.mar 500.mar CP500.mar)
 (deps    IBM500)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM850.mar 850.mar CP850.mar)
 (deps    IBM850)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM851.mar 851.mar CP851.mar)
 (deps    IBM851)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM852.mar 852.mar CP852.mar)
 (deps    IBM852)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM855.mar 855.mar CP855.mar)
 (deps    IBM855)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM856.mar 856.mar CP856.mar)
 (deps    IBM856)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM857.mar 857.mar CP857.mar)
 (deps    IBM857)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM860.mar 860.mar CP860.mar)
 (deps    IBM860)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM861.mar CP-IS.mar 861.mar CP861.mar)
 (deps    IBM861)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM862.mar 862.mar CP862.mar)
 (deps    IBM862)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM863.mar 863.mar CP863.mar)
 (deps    IBM863)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM864.mar CP864.mar)
 (deps    IBM864)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM865.mar 865.mar CP865.mar)
 (deps    IBM865)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM866.mar 866.mar CP866.mar)
 (deps    IBM866)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM866NAV.mar 866NAV.mar CP866NAV.mar)
 (deps    IBM866NAV)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM868.mar CP-AR.mar CP868.mar)
 (deps    IBM868)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM869.mar CP-GR.mar 869.mar CP869.mar)
 (deps    IBM869)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM870.mar EBCDIC-CP-YU.mar EBCDIC-CP-ROECE.mar CP870.mar)
 (deps    IBM870)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM871.mar EBCDIC-CP-IS.mar CP871.mar)
 (deps    IBM871)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM874.mar WINDOWS-874.mar CP874.mar)
 (deps    IBM874)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM875.mar EBCDIC-GREEK.mar CP875.mar)
 (deps    IBM875)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM880.mar EBCDIC-CYRILLIC.mar CP880.mar)
 (deps    IBM880)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM891.mar CP891.mar)
 (deps    IBM891)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM903.mar CP903.mar)
 (deps    IBM903)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM904.mar 904.mar CP904.mar)
 (deps    IBM904)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM905.mar EBCDIC-CP-TR.mar CP905.mar)
 (deps    IBM905)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM918.mar EBCDIC-CP-AR2.mar CP918.mar)
 (deps    IBM918)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IBM922.mar CP922.mar)
 (deps    IBM922)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IEC_P27-1.mar ISO-IR-143.mar)
 (deps    IEC_P27-1)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets INIS.mar ISO-IR-49.mar)
 (deps    INIS)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets INIS-8.mar ISO-IR-50.mar)
 (deps    INIS-8)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets INIS-CYRILLIC.mar ISO-IR-51.mar)
 (deps    INIS-CYRILLIC)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets INVARIANT.mar ISO-IR-170.mar)
 (deps    INVARIANT)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISIRI-3342.mar)
 (deps    ISIRI-3342)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO-8859-1.mar CP819.mar IBM819.mar L1.mar LATIN1.mar ISO_8859-1.mar ISO_8859-1%3A1987.mar ISO-IR-100.mar)
 (deps    ISO-8859-1)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO-8859-10.mar LATIN6.mar L6.mar ISO_8859-10.mar ISO_8859-10%3A1992.mar ISO-IR-157.mar)
 (deps    ISO-8859-10)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO-8859-11.mar)
 (deps    ISO-8859-11)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO-8859-13.mar L7.mar LATIN7.mar ISO-IR-179.mar)
 (deps    ISO-8859-13)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO-8859-14.mar L8.mar LATIN8.mar)
 (deps    ISO-8859-14)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO-8859-15.mar LATIN-9.mar ISO_8859-15.mar)
 (deps    ISO-8859-15)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO-8859-16.mar L10.mar LATIN10.mar ISO_8859-16.mar ISO_8859-16%3A2001.mar ISO-IR-226.mar)
 (deps    ISO-8859-16)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO-8859-2.mar L2.mar LATIN2.mar ISO_8859-2.mar ISO_8859-2%3A1987.mar ISO-IR-101.mar)
 (deps    ISO-8859-2)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO-8859-3.mar L3.mar LATIN3.mar ISO_8859-3.mar ISO_8859-3%3A1988.mar ISO-IR-109.mar)
 (deps    ISO-8859-3)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO-8859-4.mar L4.mar LATIN4.mar ISO_8859-4.mar ISO_8859-4%3A1988.mar ISO-IR-110.mar)
 (deps    ISO-8859-4)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO-8859-5.mar CYRILLIC.mar ISO_8859-5.mar ISO_8859-5%3A1988.mar ISO-IR-144.mar)
 (deps    ISO-8859-5)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO-8859-6.mar ARABIC.mar ASMO-708.mar ECMA-114.mar ISO_8859-6.mar ISO_8859-6%3A1987.mar ISO-IR-127.mar)
 (deps    ISO-8859-6)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO-8859-7.mar GREEK8.mar GREEK.mar ECMA-118.mar ELOT_928.mar ISO_8859-7.mar ISO_8859-7%3A1987.mar ISO_8859-7%3A2003.mar ISO-IR-126.mar)
 (deps    ISO-8859-7)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO-8859-8.mar HEBREW.mar ISO_8859-8.mar ISO_8859-8%3A1988.mar ISO-IR-138.mar)
 (deps    ISO-8859-8)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO-8859-9.mar L5.mar LATIN5.mar ISO_8859-9.mar ISO_8859-9%3A1989.mar ISO-IR-148.mar)
 (deps    ISO-8859-9)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO-8859-9E.mar)
 (deps    ISO-8859-9E)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO-IR-197.mar)
 (deps    ISO-IR-197)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO-IR-209.mar)
 (deps    ISO-IR-209)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO-IR-90.mar ISO_6937-2%3A1983.mar)
 (deps    ISO-IR-90)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO_10367-BOX.mar ISO-IR-155.mar)
 (deps    ISO_10367-BOX)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO_11548-1.mar ISO%2FTR_11548-1.mar)
 (deps    ISO_11548-1)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO_2033-1983.mar E13B.mar ISO-IR-98.mar)
 (deps    ISO_2033-1983)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO_5427.mar KOI-7.mar ISO-IR-37.mar)
 (deps    ISO_5427)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO_5427-EXT.mar ISO_5427%3A1981.mar ISO-IR-54.mar)
 (deps    ISO_5427-EXT)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO_5428.mar ISO_5428%3A1980.mar ISO-IR-55.mar)
 (deps    ISO_5428)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO_646%2EBASIC.mar ISO_646%2EBASIC%3A1983.mar REF.mar)
 (deps    ISO_646.BASIC)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO_646%2EIRV.mar IRV.mar ISO_646%2EIRV%3A1983.mar ISO-IR-2.mar)
 (deps    ISO_646.IRV)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO_6937.mar ISO6937.mar ISO_6937%3A1992.mar ISO-IR-156.mar)
 (deps    ISO_6937)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO_6937-2-25.mar ISO-IR-152.mar)
 (deps    ISO_6937-2-25)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO_6937-2-ADD.mar ISO-IR-142.mar)
 (deps    ISO_6937-2-ADD)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets ISO_8859-SUPP.mar LATIN1-2-5.mar ISO-IR-154.mar)
 (deps    ISO_8859-SUPP)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets IT.mar ISO646-IT.mar ISO-IR-15.mar)
 (deps    IT)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets JIS_C6220-1969-RO.mar ISO646-JP.mar JP.mar ISO-IR-14.mar)
 (deps    JIS_C6220-1969-RO)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets JIS_C6229-1984-B.mar JP-OCR-B.mar ISO646-JP-OCR-B.mar ISO-IR-92.mar)
 (deps    JIS_C6229-1984-B)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets JIS_X0201.mar X0201.mar)
 (deps    JIS_X0201)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets JOHAB.mar)
 (deps    JOHAB)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets JUS_I%2EB1%2E002.mar YU.mar JS.mar ISO646-YU.mar ISO-IR-141.mar)
 (deps    JUS_I.B1.002)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets JUS_I%2EB1%2E003-MAC.mar ISO-IR-147.mar MACEDONIAN.mar)
 (deps    JUS_I.B1.003-MAC)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets JUS_I%2EB1%2E003-SERB.mar SERBIAN.mar ISO-IR-146.mar)
 (deps    JUS_I.B1.003-SERB)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets KOI-8.mar)
 (deps    KOI-8)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets KOI8-R.mar)
 (deps    KOI8-R)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets KOI8-RU.mar)
 (deps    KOI8-RU)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets KOI8-T.mar)
 (deps    KOI8-T)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets KOI8-U.mar)
 (deps    KOI8-U)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets KSC5636.mar ISO646-KR.mar)
 (deps    KSC5636)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets LATIN-GREEK.mar ISO-IR-19.mar)
 (deps    LATIN-GREEK)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets LATIN-GREEK-1.mar ISO-IR-27.mar)
 (deps    LATIN-GREEK-1)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets MAC-CYRILLIC.mar)
 (deps    MAC-CYRILLIC)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets MAC-IS.mar)
 (deps    MAC-IS)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets MAC-SAMI.mar)
 (deps    MAC-SAMI)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets MAC-UK.mar)
 (deps    MAC-UK)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets MACINTOSH.mar MAC.mar)
 (deps    MACINTOSH)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets MIK.mar)
 (deps    MIK)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets MSZ_7795%2E3.mar HU.mar ISO646-HU.mar ISO-IR-86.mar)
 (deps    MSZ_7795.3)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets NATS-DANO.mar ISO-IR-9-1.mar)
 (deps    NATS-DANO)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets NATS-SEFI.mar ISO-IR-8-1.mar)
 (deps    NATS-SEFI)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets NC_NC00-10.mar ISO646-CU.mar ISO-IR-151.mar NC_NC00-10%3A81.mar CUBA.mar)
 (deps    NC_NC00-10)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets NEXTSTEP.mar NEXT.mar)
 (deps    NEXTSTEP)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets NF_Z_62-010.mar FR.mar ISO646-FR.mar ISO-IR-69.mar)
 (deps    NF_Z_62-010)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets NF_Z_62-010_%281973%29.mar ISO646-FR1.mar ISO-IR-25.mar)
 (deps    NF_Z_62-010_1973)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets NS_4551-1.mar NO.mar ISO646-NO.mar ISO-IR-60.mar)
 (deps    NS_4551-1)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets NS_4551-2.mar NO2.mar ISO-IR-61.mar ISO646-NO2.mar)
 (deps    NS_4551-2)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets PT.mar ISO646-PT.mar ISO-IR-16.mar)
 (deps    PT)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets PT154.mar Cyrillic-Asian.mar)
 (deps    PT154)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets PT2.mar ISO646-PT2.mar ISO-IR-84.mar)
 (deps    PT2)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets RK1048.mar STRK1048-2002.mar)
 (deps    RK1048)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets SAMI.mar ISO-IR-158.mar LAP.mar LATIN-LAP.mar)
 (deps    SAMI)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets WIN-SAMI-2.mar WINDOWS-SAMI2.mar WS2.mar)
 (deps    SAMI-WS2)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets SEN_850200_B.mar SS636127.mar SE.mar ISO646-SE.mar ISO646-FI.mar FI.mar ISO-IR-10.mar)
 (deps    SEN_850200_B)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets SEN_850200_C.mar SE2.mar ISO646-SE2.mar ISO-IR-11.mar)
 (deps    SEN_850200_C)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets SHIFT_JIS.mar SJIS.mar)
 (deps    SHIFT_JIS)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets SHIFT_JISX0213.mar)
 (deps    SHIFT_JISX0213)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets T%2E101-G2.mar ISO-IR-128.mar)
 (deps    T.101-G2)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets T%2E61-7BIT.mar ISO-IR-102.mar)
 (deps    T.61-7BIT)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets T%2E61-8BIT.mar ISO-IR-103.mar T%2E61.mar)
 (deps    T.61-8BIT)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets TCVN5712-1.mar TCVN5712-1%3A1993.mar TCVN-5712.mar TCVN.mar)
 (deps    TCVN5712-1)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets TIS-620.mar ISO-IR-166.mar TIS620%2E2533-0.mar TIS620%2E2529-1.mar TIS620-0.mar TIS620.mar)
 (deps    TIS-620)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets UTF-8.mar ISO-10646%2FUTF-8.mar)
 (deps    UTF-8)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets VIDEOTEX-SUPPL.mar ISO-IR-70.mar)
 (deps    VIDEOTEX-SUPPL)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets VISCII.mar)
 (deps    VISCII)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

(rule
 (targets WINDOWS-31J.mar)
 (deps    WINDOWS-31J)
 (action  (run ../tools/camomilecharmap.exe -d . %{deps})))

