let jisx4061_test1 =
  ["シャーレ";
   "シャイ";
   "シヤィ";
   "シャレ";
   "ちょこ";
   "ちよこ";
   "チョコレート";
   "てーた";
   "テータ";
   "テェタ";
   "てえた";
   "でーた"; 
   "データ";
   "デェタ";
   "でえた";
   "てーたー";
   "テータァ";
   "テェター";
   "てぇたぁ";
   "てえたー";
   "でーたー";
   "データァ";
   "でェたァ";
   "デぇタぁ";
   "デエタア";
   "ひゆ";
   "びゅあ";
   "ぴゅあ";
   "びゅあー";
   "ビュアー";
   "ぴゅあー";
   "ピュアー";
   "ヒュウ";
   "ヒユウ";
   "ピュウア";
   "びゅーあー"; 
   "ビューアー";
   "ビュウアー";
   "ひゅん";
   "ぴゅん";
   "ふーり";
   "フーリ";
   "ふぅり";
   "ふゥり";
   "ふゥリ";
   "フウリ";
   "ぶーり";
   "ブーリ";
   "ぶぅり";
   "ブゥり";
   "ぷうり";
   "プウリ";
   "ふーりー";
   "フゥリー";
   "ふゥりィ";
   "フぅりぃ";
   "フウリー";
   "ふうりぃ";
   "ブウリイ";
   "ぷーりー";
   "ぷゥりイ";
   "ぷうりー";
   "プウリイ";
   "フヽ";
   "ふゞ";
   "ぶゝ";
   "ぶふ";
   "ぶフ";
   "ブふ";
   "ブフ";
   "ぶゞ";
   "ぶぷ";
   "ブぷ";
   "ぷゝ";
   "プヽ";
   "ぷふ";]

let jisx4061_test2 =
  ["はゝ"; 
   "はは"; 
   "はハ"; 
   "ハハ"; 
   "はゞ"; 
   "ハヾ"; 
   "はば"; 
   "ハバ"; 
   "はぱ"; 
   "ハぱ"; 
   "ハパ"; 
   "ばゝ"; 
   "バヽ"; 
   "ばは"; 
   "バハ"; 
   "ばゞ"; 
   "バヾ"; 
   "ばば"; 
   "バば"; 
   "ババ"; 
   "ばぱ"; 
   "バパ"; 
   "ぱゝ"; 
   "パヽ"; 
   "ぱは"; 
   "パハ"; 
   "ぱば"; 
   "ぱバ"; 
   "パバ"; 
   "ぱぱ"; 
   "パパ";]

let test_desc_1 = "test_1 i <<< い << ii = i*"
let test_list_1 = ["bbbbiccc"; "bbbbいccc"; "bbbbiiccc"; "bbbbi*dcc"]

let test_desc_2 = "test_1 &[before 3]I <<< イ"
let test_list_2 =     ["aaaahbbbb"; "aaaaイbbbb"; "aaaaIbbbb"]

let test_desc_3 =  "test_1 &[before 3]UE <<< ウェ <<< UE <<< ue"
let test_list_3 =     ["aaaaudbbbb";
                       "aaaaウェbbbb"; 
                       "aaaaUEbbbb"; 
                       "aaaauebbbb";
                       "aaaaUFbbbb"]

let test_desc_4 = "test_1 &[before 3]V <<< ヴェ/ェ = V <<< v"
let test_list_4 =      ["aaaaVェabbb";
                        "aaaaヴェbbbb";
                        "aaaaVェcbbb";
                        "aaaavェcbbb"]

let test_desc_5 = "test_1 &皇帝 < emperor"
let test_list_5 =     ["anon_皇帝";
                       "anon_emperor"]


